import os
from unittest import TestCase
from requirements_detector.detect import from_setup_py, CouldNotParseRequirements


class SyntaxErrorTest(TestCase):

    def test_setup_py_syntax_error(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/syntax_error/setup.py')
        self.assertRaises(CouldNotParseRequirements, from_setup_py, filepath)

    def test_setup_py_multiline_string(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/syntax_error/setup_multiline_string.py')
        from_setup_py(filepath)

    def test_regular_indentation(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/syntax_error/regular_indentation.py')
        from_setup_py(filepath)



from distutils.core import setup

_install_requires = [
    'Django==1.5.0',
    'django-gubbins==1.1.2'
]

setup(
    name='prospector-test-2',
    version='0.0.1',
    install_requires=_install_requires
)


from distutils.core import setup

setup(
    name='prospector-test-1',
    version='0.0.1',
    requires=[
        'Django==1.5.0',
    ],
    install_requires=[
        'django-gubbins==1.1.2'
    ]
)


import os
from unittest import TestCase
from requirements_detector.detect import from_requirements_txt, from_requirements_dir, \
    from_requirements_blob, from_setup_py, CouldNotParseRequirements
from requirements_detector.requirement import DetectedRequirement


class DependencyDetectionTest(TestCase):

    def _expected(self, *requirements):
        return [DetectedRequirement.parse(req) for req in requirements]

    def test_requirements_txt_parsing(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test1/requirements.txt')
        dependencies = from_requirements_txt(filepath)

        expected = self._expected(
            'amqp!=1.0.13',
            'Django>=1.5.0',
            'six<1.4,>=1.3.0',
            'South==0.8.2',
        )

        self.assertEqual(expected, sorted(dependencies))

    def test_requirements_dir_parsing(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test2/requirements')
        dependencies = from_requirements_dir(filepath)

        expected = self._expected(
            'amqp==1.0.13',
            'anyjson==0.3.3',
            'Django==1.5.2',
            'South==0.8.2',
        )

        self.assertEqual(expected, sorted(dependencies))

    def test_requirements_blob_parsing(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test3')
        dependencies = from_requirements_blob(filepath)

        expected = self._expected(
            'amqp==1.0.13',
            'anyjson==0.3.3',
            'django-gubbins==1.1.2',
        )

        self.assertEqual(expected, sorted(dependencies))

    def test_invalid_requirements_txt(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test5/invalid_requirements.txt')
        dependencies = from_requirements_txt(filepath)
        expected = self._expected('django<1.6', 'django')
        self.assertEqual(expected, sorted(dependencies))

    def test_invalid_requirements_txt(self):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test6/requirements.txt')
        from_requirements_txt(filepath)

    def _test_setup_py(self, setup_py_file, *expected):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test4', setup_py_file)
        dependencies = from_setup_py(filepath)
        expected = self._expected(*expected)
        self.assertEqual(expected, sorted(dependencies))

    def _test_setup_py_not_parseable(self, setup_py_file):
        filepath = os.path.join(os.path.dirname(__file__), 'detection/test4', setup_py_file)
        self.assertRaises(CouldNotParseRequirements, from_setup_py, filepath)

    def test_simple_setup_py_parsing(self):
        self._test_setup_py('simple.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_setup_py_reqs_defined_in_file_parsing(self):
        self._test_setup_py('in_file.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_setup_py_tuple(self):
        self._test_setup_py('tuple.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_subscript_assign(self):
        self._test_setup_py('subscript_assign.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_utf8_setup_py(self):
        self._test_setup_py('utf8.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_requires_setup_py(self):
        self._test_setup_py('uses_requires.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_requires_and_install_requires_setup_py(self):
        self._test_setup_py('uses_requires_and_install_requires.py', 'Django==1.5.0', 'django-gubbins==1.1.2')

    def test_callable_install_requires(self):
        self._test_setup_py_not_parseable('callable.py')


from distutils.core import setup

def _install_requires():
    return [
        'Django==1.5.0',
        'django-gubbins==1.1.2'
    ]

setup(
    name='prospector-test-2',
    version='0.0.1',
    install_requires=_install_requires()
)


# -*- coding: UTF-8 -*-
from distutils.core import setup

if foo:
    # just for a test with indentation
    bar()

setup(
    name=u'prospector-test-4-üéø',
    version='0.0.1',
    install_requires=[
        'Django==1.5.0',
        'django-gubbins==1.1.2'
    ]
)



try:
    import urlparse
except ImportError:
    # python3
    from urllib import parse as urlparse

from unittest import TestCase
from requirements_detector.requirement import DetectedRequirement, _parse_egg_name, _strip_fragment


class TestRequirementParsing(TestCase):

    def _test(self, requirement, name=None, version_specs=None, url=None):
        req = DetectedRequirement.parse(requirement)
        self.assertEqual(name, req.name)
        if version_specs is None:
            self.assertEqual([], req.version_specs)
        else:
            for spec in version_specs:
                self.assertTrue(spec in req.version_specs)
        self.assertEqual(url, req.url)

    def test_basic_requirement(self):
        self._test('Django', 'django')
        self._test('celery', 'celery')

    def test_requirement_with_versions(self):
        self._test('Django==1.5.2', 'django', [('==', '1.5.2')])
        self._test('South>0.8', 'south', [('>', '0.8')])
        self._test('django-gubbins!=1.1.1,>1.1', 'django-gubbins', [('!=', '1.1.1'), ('>', '1.1')])

    def test_relative_file_path(self):
        self._test('../somelib', url='../somelib')

    def test_vcs_url(self):
        self._test('git+ssh://git@github.com/something/somelib.git',
                   url='git+ssh://git@github.com/something/somelib.git')
        self._test('git+ssh://git@github.com/something/somelib.git#egg=somelib',
                   name='somelib', url='git+ssh://git@github.com/something/somelib.git')
        self._test('git://github.com/peeb/django-mollie-ideal.git#egg=mollie',
                   name='mollie', url='git+git://github.com/peeb/django-mollie-ideal.git')

    def test_archive_url(self):
        self._test('http://example.com/somelib.tar.gz', url='http://example.com/somelib.tar.gz')
        self._test('http://example.com/somelib.tar.gz#egg=somelib', name='somelib',
                   url='http://example.com/somelib.tar.gz')

    def test_editable_relative_path(self):
        self._test('-e ../somelib', url='../somelib')

    def test_editable_vcs_url(self):
        self._test('--editable git+ssh://git@github.com/something/somelib.git#egg=somelib',
                   name='somelib', url='git+ssh://git@github.com/something/somelib.git')


class TestEggFragmentParsing(TestCase):

    def test_simple(self):
        self.assertEqual('somelib', _parse_egg_name('egg=somelib'))

    def test_no_egg_value(self):
        self.assertTrue(_parse_egg_name('a=b&c=2') is None)

    def test_no_pairs(self):
        self.assertTrue(_parse_egg_name('somelib') is None)

    def test_first_egg_val(self):
        self.assertEqual('somelib', _parse_egg_name('egg=somelib&egg=anotherlib'))

    def test_multiple_fragment_values(self):
        self.assertEqual('somelib', _parse_egg_name('a=1&egg=somelib&b=2'))


class TestFragmentStripping(TestCase):

    def test_stripping(self):
        url = 'http://example.com/index.html?a=b&c=2#some_fragment'
        parts = urlparse.urlparse(url)
        self.assertEqual('http://example.com/index.html?a=b&c=2', _strip_fragment(parts))



"""
This test is to verify that top level subscript assigns (x[y]) don't break the
parser. For version <=0.1, a subscript assign would break the setup.py AST walker
completely.
"""

from distutils.core import setup

something = dict()
something['fish'] = ['a', 'b', 'c']

setup(
    name='prospector-test-1',
    version='0.0.1',
    install_requires=(
        'Django==1.5.0',
        'django-gubbins==1.1.2'
    )
)